/*
****************************************************************************
****************************************************************************
**                                                                        **
**                XXXX     X    X         X XXXXXX X                      **
**               X    X   X X   X    X    X X      X   X                  **
**               X       X   X  X   X X   X X      X  X                   **
**               X   XX XXXXXXX X  X   X  X XXXX   X X                    **
**               X    X X     X X X     X X X      XX                     **
**               X    X X     X XX       XX X      X                      **
**                XXXX  X     X X         X XXXXXX XXXXXX                 **
**                                                                        **
**                                                                        **
**                           BIALYSTOK 2018                               **
**                                                                        **
**                          MODULE XRAMTester                             **
**                                                                        **
**                     Mikrokontroler ARM: LPC2378                        **
**                                                                        **
**                     Wersja z dnia : 2018.06.12                         **
**                     Napisal : Andrzej Pawluczuk                        **
**                                                                        **
****************************************************************************
****************************************************************************
*/

#include <LPC23xx.H>
#include "system.h"
#include "types.h"
#include "irqvectors.h"
#include "serialconst.h"
#include "uart0.h"
#include "uart1.h"
#include "strings.h"

#define IRQCallCntLimit                 100

#define XRAMBank0BaseAddress            0x80000000
#define XRAMBank1BaseAddress            0x81000000

static ULONG IRQCallCounter ;


static __irq void T1_IRQHandler ( void )
{
  /*-------------------------------------------------------------------------*/
  IRQCallCounter ++ ;
  if ( IRQCallCounter >= IRQCallCntLimit )
  {
    IRQCallCounter = 0 ;
  } /* if */ ;
  T1IR = 1 ;
  VICVectAddr = 0 ;
} /* T1_IRQHandler */


#define PCEMC               0x00000800  /* PCEMC External Memory Controller. */

static void InitXRAM ( void )
{
  /*-------------------------------------------------------------------------*/
  PCONP |= PCEMC ;
  PINSEL6 = 0x00005555 ; /* D0 .. D7 */
  PINSEL8 = 0x55555555 ; /* A0 .. A15 */
  PINSEL9 = 0x50090000 ; /* OE, BLS0, CS0, CS1 */
} /* InitXRAM */


static void HardwareInit ( void ) 
{
  /*-------------------------------------------------------------------------*/
  InitXRAM ( ) ;
  PINSEL10 = 0 ;
  T1MR0 = ( PCLK / 1000 ) - 1 ;
  T1MCR = 3 ;
  T1TCR = 1 ;
  VICVectAddr5 = ( unsigned long ) T1_IRQHandler ;
  VICVectCntl5 = 15 ;
  VICIntEnable = ( 1 << TIMER1_Interrupt ) ;
  UART0InitEnvir ( ) ;
  UART1InitEnvir ( ) ;
} /* HardwareInit */


static void TraceReg ( UCHAR * Prompt , ULONG Data )
{
  UCHAR Number [ 16 ] ;
  /*-------------------------------------------------------------------------*/
  UART1SendString ( Prompt ) ;
  LongToHexStr ( Number , 8 , '0' , Data ) ;
  UART1SendString ( Number ) ;
  UART1Send ( ( UCHAR ) '\r' ) ;
  UART1Send ( ( UCHAR ) '\n' ) ;
} /* Trace Reg */


UCHAR Hello1 [ ]      = "\r\nHello: xram tester\r\n" ;
UCHAR PCONPTxt [ ]    = "PCONP=" ;
UCHAR PINSEL0Txt [ ] = "PINSEL0=" ;
UCHAR PINSEL1Txt [ ] = "PINSEL1=" ;
UCHAR PINSEL2Txt [ ] = "PINSEL2=" ;
UCHAR PINSEL3Txt [ ] = "PINSEL3=" ;
UCHAR PINSEL4Txt [ ] = "PINSEL4=" ;
UCHAR PINSEL5Txt [ ] = "PINSEL5=" ;
UCHAR PINSEL6Txt [ ] = "PINSEL6=" ;
UCHAR PINSEL7Txt [ ] = "PINSEL7=" ;
UCHAR PINSEL8Txt [ ] = "PINSEL8=" ;
UCHAR PINSEL9Txt [ ] = "PINSEL9=" ;
UCHAR PINSEL10Txt [ ] = "PINSEL10=" ;

static void HelloMessage ( void )
{
  /*-------------------------------------------------------------------------*/
  UART1SendString ( Hello1 ) ;
} /* HelloMessage */


static void RegInfo ( void )
{
  /*-------------------------------------------------------------------------*/
  TraceReg ( PCONPTxt , PCONP ) ;
  TraceReg ( PINSEL0Txt , PINSEL0 ) ;
  TraceReg ( PINSEL1Txt , PINSEL1 ) ;
  TraceReg ( PINSEL2Txt , PINSEL2 ) ;
  TraceReg ( PINSEL3Txt , PINSEL3 ) ;
  TraceReg ( PINSEL4Txt , PINSEL4 ) ;
  TraceReg ( PINSEL5Txt , PINSEL5 ) ;
  TraceReg ( PINSEL6Txt , PINSEL6 ) ;
  TraceReg ( PINSEL7Txt , PINSEL7 ) ;
  TraceReg ( PINSEL8Txt , PINSEL8 ) ;
  TraceReg ( PINSEL9Txt , PINSEL9 ) ;
  TraceReg ( PINSEL10Txt , PINSEL10 ) ;
} /* RegInfo */


static void SoftwareInit ( void )
{
  /*-------------------------------------------------------------------------*/
  IRQCallCounter = 0 ;
} /* SoftwareInit */


UCHAR FillRAMTxt [ ] = "\r\nZapis do pamieci: bank " ;

void Fill ( ULONG BaseAdress )
{
  UCHAR * Ptr ;
  UCHAR Data ;
  USHORT Loop ;
  UCHAR Number [ 16 ] ;
  /*-------------------------------------------------------------------------*/
  UART1SendString ( FillRAMTxt ) ;
  LongToHexStr ( Number , 8 , '0' , BaseAdress ) ;
  UART1SendString ( Number ) ;
  UART1Send ( ( UCHAR ) '\r' ) ;
  UART1Send ( ( UCHAR ) '\n' ) ;
  Ptr = ( UCHAR * ) BaseAdress ;
  Data = 0 ;
  for ( Loop = 0 ; Loop < 16000 ; Loop ++ )
  {
    * Ptr ++ = Data ++ ;
  } /* loop */ ;
//  Ptr = ( UCHAR * ) ( BaseAdress + 0x123 ) ; // Symulacja bledu pamieci XRAM
//  * Ptr = 0xFF ;                             // Symulacja bledu pamieci XRAM
} /* Fill */

UCHAR CheckRAMTxt  [ ] = "\r\nSprawdzenie zawartosci pamieci: bank " ;
UCHAR RAMErrorTxt  [ ] = "Blad w pamieci RAM na adresie:" ;
UCHAR RAMError1Txt [ ] = " oczekiwane:" ;
UCHAR RAMError2Txt [ ] = " rzeczywiste:" ;
UCHAR RamTestOKTxt [ ] = "\r\nNie stwierdzono problemow w pamieci XRAM\r\n" ;

void Check ( ULONG BaseAdress )
{
  UCHAR * Ptr ;
  UCHAR ReqData ;
  UCHAR RealData ;
  USHORT Loop ;
  UCHAR Number [ 16 ] ;
  UCHAR ErrorFlag ;
  /*-------------------------------------------------------------------------*/
  UART1SendString ( CheckRAMTxt ) ;
  LongToHexStr ( Number , 8 , '0' , BaseAdress ) ;
  UART1SendString ( Number ) ;
  UART1Send ( ( UCHAR ) '\r' ) ;
  UART1Send ( ( UCHAR ) '\n' ) ;
  ErrorFlag = FALSE ;
  Ptr = ( UCHAR * ) BaseAdress ;
  ReqData = 0 ;
  for ( Loop = 0 ; Loop < 16000 ; Loop ++ )
  {
    RealData = * Ptr ;
    if ( RealData != ReqData )
    {
      UART1SendString ( RAMErrorTxt ) ;
      LongToHexStr ( Number , 8 , '0' , ( ULONG ) Ptr ) ;
      UART1SendString ( Number ) ;
      UART1SendString ( RAMError1Txt ) ;
      WordToHexStr ( Number , 2 , '0' , ( USHORT ) ReqData ) ;
      UART1SendString ( Number ) ;
      UART1SendString ( RAMError2Txt ) ;
      WordToHexStr ( Number , 2 , '0' , ( USHORT ) RealData ) ;
      UART1SendString ( Number ) ;
      UART1Send ( ( UCHAR ) '\r' ) ;
      UART1Send ( ( UCHAR ) '\n' ) ;
      ErrorFlag = TRUE ;
    } /* if */ ;
    ReqData ++ ;
    Ptr ++ ;
  } /* loop */ ;
  if ( ! ErrorFlag )
    UART1SendString ( RamTestOKTxt ) ;
} /* Check */


int main ( void )
{
  /*-------------------------------------------------------------------------*/
  SysInit ( ) ;
  HardwareInit ( )  ;
  SoftwareInit ( ) ;
  UART0HardwInit( FOSC , N81Mode , 9600 ) ;
  UART1HardwInit( FOSC , N81Mode , 9600 ) ;
  HelloMessage ( ) ;
  RegInfo ( ) ;
  Fill ( XRAMBank0BaseAddress ) ;
  Check ( XRAMBank0BaseAddress ) ;
  Fill ( XRAMBank1BaseAddress ) ;
  Check ( XRAMBank1BaseAddress ) ;
  for ( ; ; )
  {
  } /* loop */ ;
} /* main */

